require 'active_record'
require 'genome-lib'
require 'fileutils'

Signal.trap("TERM") do
  # Clean up any active record connections
  ActiveRecord::Base.remove_connection

  Cloudhost::VirtWatcher.stop

  # Cleanly exit
  exit 0
end

module Cloudhost
  class VirtWatcher
    include GenomeLib
     
    def initialize
      @virsh = Virsh.new
      @guests = []
    end
    
    def collect_guests
      @guests = @virsh.list(:all => true)
      $LOG.debug("Found #{@guests.size} guests.")

      @guests.each do |g|
        guest = Cloudhost::Models::Guest.find(:first, :conditions => "uuid = '#{g.uuid}'")
        if guest.nil?
          $LOG.debug("Creating a new guest: #{g.name}, #{g.uuid}, #{g.status}")
          Cloudhost::Models::Guest.create :name => g.name,
                       :uuid => g.uuid,
                       :status => g.status,
                       :memsize => g.memsize,
                       :last_updated => Time.now
        else
          $LOG.debug("Guest with uuid '#{g.uuid}' found.")
          # This should handle the case of a guest being renamed
          guest.name = g.name
          guest.status = g.status
          guest.memsize = g.memsize
          guest.last_updated = Time.now
          guest.save!
        end
      end
    end

    def cleanup_stale_autostart_files
      xen_dir = "/etc/xen/auto"
      valid_files = @guests.map {|g| File.join(xen_dir, g.name)}
      all_files = Dir[File.join(xen_dir, "*")]

      (all_files - valid_files).each do |f| 
        $LOG.debug("Cleaning up stale autostart file: #{f}")
        FileUtils.rm_f(f)
      end
    end

    # The guests name must == the autostart file name
    def autostart_guests
      running = @guests.find_all {|g| g.running?}.map {|g| g.name}
      auto_dir = xen_host? ? "/etc/xen/auto" : "/etc/libvirt/qemu/autostart"
      # Rip off the extension for KVM guests
      autostartable = Dir[File.join(auto_dir, "*")].map {|f| File.basename(f, ".xml")}

      # Only start domains that aren't already running
      (autostartable - running).each do |g| 
        $LOG.debug("Starting #{g}")
        IO.popen("virsh start #{g} 2>&1") do |io|
          $LOG.debug(io.read)
        end

        if $?.exitstatus != 0
           guest = Cloudhost::Models::Guest.find(:first, :conditions => "name = '#{g}'")
           guest.status = "failed"
           guest.save!
        end
      end
    end

    def xen_host?
      File.exist?("/proc/xen")
    end

    def run
      collect_guests
      cleanup_stale_autostart_files if xen_host?
      autostart_guests
    end
  end
end
